@@ -2,7 +2,7 @@ import pprint
 from contextlib import ExitStack
 from distutils.version import StrictVersion
 from enum import Enum
-from typing import Optional, Union, Generator
+from typing import Optional, Union, Generator, Any
 from unittest import mock
 
 import pytest
@@ -11,57 +11,43 @@ import pytest
 __all__ = ['mock_hedwig_publish']
 
 
-class AnyDict(dict):
-    """An object equal to any dict."""
-
-    def __eq__(self, other):
-        return isinstance(other, dict)
-
-    def __repr__(self):
-        return f'{type(self).__name__}()'
-
-
 class HedwigPublishMock(mock.MagicMock):
     """
     Custom mock class used by :meth:`hedwig.testing.pytest_plugin.mock_hedwig_publish` to mock the publisher.
     """
 
     def _message_published(
-        self, msg_type: Union[str, Enum], data: Optional[dict], version: Union[str, StrictVersion]
+        self, msg_type: Union[str, Enum], data: Optional[Any], version: Union[str, StrictVersion]
     ) -> bool:
         if isinstance(msg_type, Enum):
             msg_type = msg_type.value
         return any(
-            msg.type == msg_type and msg.data == data and msg.data_schema_version == version
+            msg.type == msg_type and (data is None or msg.data == data) and msg.version == version
             for (msg,), _ in self.call_args_list
         )
 
     def _error_message(self) -> str:
-        return pprint.pformat([(msg.type, msg.data, msg.data_schema_version) for (msg,), _ in self.call_args_list])
+        return pprint.pformat([(msg.type, msg.data, msg.version) for (msg,), _ in self.call_args_list])
 
     def assert_message_published(
-        self, msg_type: Union[str, Enum], data=None, version: Union[str, StrictVersion] = StrictVersion('1.0')
+        self, msg_type: Union[str, Enum], data: Any = None, version: Union[str, StrictVersion] = StrictVersion('1.0')
     ) -> None:
         """
         Helper function to check if a Hedwig message with given type, data
         and schema version was sent.
         """
-        if data is None:
-            data = AnyDict()
         if not isinstance(version, StrictVersion):
             version = StrictVersion(version)
 
         assert self._message_published(msg_type, data, version), self._error_message()
 
     def assert_message_not_published(
-        self, msg_type: Union[str, Enum], data=None, version: Union[str, StrictVersion] = StrictVersion('1.0')
+        self, msg_type: Union[str, Enum], data: Any = None, version: Union[str, StrictVersion] = StrictVersion('1.0')
     ) -> None:
         """
         Helper function to check that a Hedwig message of given type, data
         and schema was NOT sent.
         """
-        if data is None:
-            data = AnyDict()
         if not isinstance(version, StrictVersion):
             version = StrictVersion(version)
 
